/************************************************************
*  pc38.c - Device driver for Oregon Microsystem's pc38.
*  NASA IRTF
*  2680 Woodlawn Drive, Honolulu, HI, 96822
*
*  Author: Tony Denault
*************************************************************/

#include <conf.h>
#include <kernel.h>
#include <file.h>
#include <dldd.h>

#include "/sys/dheaders/pc38info.h"
#include "proto.h"

extern char *sysbrk();
extern int  sysfree();

/*--------------------------------------------
** structure of all statics the driver needs
*/
struct stuff {
   int addr;                /* address of PC-38 card      */
   int iosem;               /* sem to command io */
   int df;                  /* done flag bits stored here */
};

/************************************************************
             Devide Driver Interface
************************************************************/

/*----------------------------------------------------------
** Open the device driver
**----------------------------------------------------------
*/
int pc38open( s, d, f)
   struct stuff *s;            /* driver data   */
   int d;                      /* device number */
   struct file * f;            /* file descriptor */
{
   return (OK);
}

/*----------------------------------------------------------
**  Close the device driver
**----------------------------------------------------------
*/
int pc38close()
{
   return (OK);
}

/*----------------------------------------------------------
**  Read from the device
**----------------------------------------------------------
*/
int pc38read( s, f, buff, count)
   struct stuff *s;
   struct file *f;
   char *buff;
   register int count;
{
   return 0;
}

/*----------------------------------------------------------
**  Write to the device driver
**----------------------------------------------------------
*/
int pc38write( s, f, buff, count)
   struct stuff *s;
   struct file *f;
   char *buff;
   register int count;
{
   return (count);
}

/*----------------------------------------------------------
**  Device control
**----------------------------------------------------------
*/
int pc38ioctl( s, f, command, pc38p )
   struct stuff *s;
   struct file *f;
   int command;
   struct pc38parms_t *pc38p;
{
   /* read done flag copy set bits into df */
   s->df |= PC38_inp8(s->addr+PC38_DONE_REG);
   pc38p->df = s->df;

   /* process user's request */
   switch( command )
   {
      case PC38GET_SETUP:
         pc38p->ret = ERR_NONE;
         pc38p->i1  = s->addr;
         break;

      case PC38PUT_SETUP:
         pc38p->ret = ERR_NONE;
         s->addr = pc38p->i1;
         break;

      case PC38DF_GET:
         pc38p->ret = ERR_NONE;
         break;

      case PC38DF_CLEAR:
         pc38p->ret = ERR_NONE;
         s->df &= ~(0x0ff & pc38p->i1);
         pc38p->df = s->df;
         break;

      case PC38PUTS:
         swait( &s->iosem, -1);
         pc38p->ret = PC38_puts( s->addr, pc38p->p1);
         ssignal( &s->iosem );
         break;

      case PC38GETS:
         swait( &s->iosem, -1);
         pc38p->ret = PC38_gets( s->addr, pc38p->p1, pc38p->i1);
         ssignal( &s->iosem );
         break;

      case PC38COMMAND:
         swait( &s->iosem, -1);
         pc38p->ret = PC38_command( s->addr, pc38p->p1, pc38p->p2, pc38p->i1);
         ssignal( &s->iosem );
         break;

      case PC38GET_STATUS:
         pc38p->i1 = PC38_inp8(s->addr+PC38_CONTROL_REG);
         pc38p->i2 = PC38_inp8(s->addr+PC38_STATUS_REG);
         pc38p->ret = ERR_NONE;
         break;

      case PC38TEST:
         PC38csleep( (int) pc38p->i1 );
         pc38p->ret = ERR_NONE;
         break;

      case PC38_OUTP8:
         PC38_outp8( pc38p->i1, pc38p->i2);
         pc38p->ret = ERR_NONE;
         break;
 
      case PC38_INP8:
         pc38p->i2 = PC38_inp8( pc38p->i1 );
         pc38p->ret = ERR_NONE;
         break;
 
      default:
         pc38p->ret = ERR_INV_FUN;
         break;
   }

   return (OK);
}

/*----------------------------------------------------------
**  Selection
**----------------------------------------------------------
*/
int pc38select()
{
   cprintf("pc38select()\n");
   return (OK);
}

/*----------------------------------------------------------
**  Initial device installation. Allocates memory for
**  pc38info.
**----------------------------------------------------------
*/
struct stuff * pc38install( info )
   struct pc38info_t * info;
{
   struct stuff *s;

   cprintf("pc38install() \n" );
   s = (struct stuff *)sysbrk((long)sizeof(struct stuff));
   s->addr = info->addr;
   s->iosem = 1;
   s->df = 0;
   return s;
}

/*----------------------------------------------------------
**  Device un-installation
**----------------------------------------------------------
*/
int pc38uninstall( s )
   struct stuff *s;
{
   cprintf("pc38uninstall()\n");
   sysfree( s, (long)sizeof(struct stuff));
   return (OK);
}

/*  Declare Entry point for dynamically loadable Device Drivers */

/*static struct dldd entry_points = { pc38open, pc38close, pc38read,
**   pc38write, pc38select, pc38ioctl, pc38install, pc38uninstall };
*/

/*---------------------------- eof ------------------------------*/
